@@ -3,7 +3,7 @@ use std::ops;
 use crate::document::Document;
 use crate::key::Key;
 use crate::table::TableKeyValue;
-use crate::{value, InlineTable, Item, Table, Value};
+use crate::{value, InlineTable, InternalString, Item, Table, Value};
 
 // copied from
 // https://github.com/serde-rs/json/blob/master/src/value/index.rs
@@ -56,24 +56,23 @@ impl Index for str {
         }
     }
     fn index_or_insert<'v>(&self, v: &'v mut Item) -> &'v mut Item {
-        let key = Key::new(self);
         if let Item::None = *v {
             let mut t = InlineTable::default();
             t.items.insert(
-                key.get().to_owned(),
-                TableKeyValue::new(key.clone(), Item::None),
+                InternalString::from(self),
+                TableKeyValue::new(Key::new(self), Item::None),
             );
             *v = value(Value::InlineTable(t));
         }
         match *v {
-            Item::Table(ref mut t) => t.entry(key.get()).or_insert(Item::None),
+            Item::Table(ref mut t) => t.entry(self).or_insert(Item::None),
             Item::Value(ref mut v) if v.is_inline_table() => {
                 &mut v
                     .as_inline_table_mut()
                     .unwrap()
                     .items
-                    .entry(key.get().to_owned())
-                    .or_insert(TableKeyValue::new(key, Item::None))
+                    .entry(InternalString::from(self))
+                    .or_insert_with(|| TableKeyValue::new(Key::new(self), Item::None))
                     .value
             }
             _ => panic!("cannot access key {}", self),
